from unittest import TestCase

from ha_services.mqtt4homeassistant.components.select import Select
from ha_services.mqtt4homeassistant.data_classes import ComponentConfig, ComponentState
from ha_services.mqtt4homeassistant.device import MqttDevice


class SelectTestCase(TestCase):
    def test_select(self):
        select = Select(
            device=MqttDevice(name='My Device', uid='device1'),
            name='My Select',
            uid='select1',
            options=('Option 1', 'Option 2'),
            default_option='Option 1',
        )
        self.assertEqual(
            select.get_config(),
            ComponentConfig(
                topic='homeassistant/select/device1/device1-select1/config',
                payload={
                    'command_topic': 'homeassistant/select/device1/device1-select1/command',
                    'component': 'select',
                    'device': {'identifiers': 'device1', 'name': 'My Device'},
                    'json_attributes_topic': 'homeassistant/select/device1/device1-select1/attributes',
                    'name': 'My Select',
                    'options': ('Option 1', 'Option 2'),
                    'state_topic': 'homeassistant/select/device1/device1-select1/state',
                    'unique_id': 'device1-select1',
                },
            ),
        )
        self.assertEqual(
            select.get_state(),
            ComponentState(topic='homeassistant/select/device1/device1-select1/state', payload='Option 1'),
        )
        with self.assertRaises(AssertionError) as cm:
            select.set_state('invalid_state')
        self.assertEqual(
            str(cm.exception),
            "Invalid state: 'invalid_state' - Existing options: ('Option 1', 'Option 2')",
        )

        select.set_state('Option 2')
        self.assertEqual(
            select.get_state(),
            ComponentState(topic='homeassistant/select/device1/device1-select1/state', payload='Option 2'),
        )



import json
from unittest import TestCase

from bx_py_utils.test_utils.snapshot import assert_snapshot

from ha_services.mqtt4homeassistant.device import MqttDevice
from ha_services.mqtt4homeassistant.mocks import HostSystemMock
from ha_services.mqtt4homeassistant.mocks.mqtt_client_mock import MqttClientMock
from ha_services.mqtt4homeassistant.system_info.netstat import NetStatSensor, NetStatSensors


class NetStatSensorsTestCase(TestCase):

    def test_happy_path(self):
        with HostSystemMock(), self.assertLogs('ha_services', level='DEBUG'):
            netstat_sensors = NetStatSensors(
                device=MqttDevice(name='foo', uid='bar'),
            )
            sensor = netstat_sensors.sensors['eth0']
            self.assertIsInstance(sensor, NetStatSensor)
            self.assertEqual(
                sensor.bytes_sent_sensor.get_state().topic,
                'homeassistant/sensor/bar/bar-eth0sent/state',
            )

            mqtt_client_mock = MqttClientMock()
            netstat_sensors.publish(mqtt_client_mock)
        topics = [message['topic'] for message in mqtt_client_mock.messages]
        self.assertEqual(
            topics,
            [
                'homeassistant/sensor/bar/bar-eth0sent/config',
                'homeassistant/sensor/bar/bar-eth0sent/state',
                'homeassistant/sensor/bar/bar-eth0sentrate/config',
                'homeassistant/sensor/bar/bar-eth0sentrate/state',
                'homeassistant/sensor/bar/bar-eth0received/config',
                'homeassistant/sensor/bar/bar-eth0received/state',
                'homeassistant/sensor/bar/bar-eth0receivedrate/config',
                'homeassistant/sensor/bar/bar-eth0receivedrate/state',
            ],
        )
        configs = [
            json.loads(message['payload']) for message in mqtt_client_mock.messages if '/config' in message['topic']
        ]
        assert_snapshot(got=configs)



from paho.mqtt.client import Client


class MqttClientMock(Client):
    def __init__(self):
        self.messages = []

    def publish(self, **kwargs) -> None:
        self.messages.append(kwargs)

    def _reset_sockets(self):
        pass
